﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

#if !UNITY_WEBGL && !NET_2_0_SUBSET

using MySql.Data;
using MySql.Data.MySqlClient;

#endif

public class RTSqlManager : MonoBehaviour
{

#if !UNITY_WEBGL && !NET_2_0_SUBSET

    const float _heartBeatTimerIntevalSeconds = (60*60); //an hour. a good ouwah

    static RTSqlManager _this = null;
    MySqlConnection _conn = null;
    string _dataBaseName = "";
    string _serverIP = "";
    string _userName = "";
    string _userPassword = "";

    float _AntiTimeoutTimer = 0;

    public enum SqlResult
    {
        ERROR = -99
    }
      
    static public RTSqlManager Get() { return _this; }

    void Awake()
    {
        _this = this;
    }
    public static string GetName()
    {
        return Get().name;
    }

    public string EscapeString(string str)
    {
        return MySql.Data.MySqlClient.MySqlHelper.EscapeString(str);
    }

    private void OnDestroy()
    {
        if (_conn != null)
        {
            _conn.Close();
        }
        print("MYSQL destroyed");
    }

    void DoEmergencyReconnect()
    {
        RTConsole.Log("Attempting emergency reconnect to SQL...");
        _conn = new MySqlConnection("SERVER=" + _serverIP + ";DATABASE=" + _dataBaseName + ";UID=" + _userName + ";PASSWORD=" + _userPassword + ";");
        _conn.Open();
    }

    public bool Init(string serverIP, string userName, string userPassword, string databaseName)
    {
        _dataBaseName = databaseName;
        _serverIP = serverIP;
        _userName = userName;
        _userPassword = userPassword;


        _conn = new MySqlConnection("SERVER=" + _serverIP + ";DATABASE=" + _dataBaseName + ";UID=" + _userName + ";PASSWORD=" + _userPassword + ";");
        _conn.Open();
        return _conn != null;
    }

    public bool DoesTableExist(string tableName)
    {
        string cmdStr = "SELECT COUNT(*) FROM information_schema.tables WHERE table_schema = '"+ _dataBaseName+"' AND table_name = '"+ tableName+"'";
        MySqlCommand cmd = new MySqlCommand(cmdStr, _conn);
        MySqlDataReader reader = cmd.ExecuteReader();

        bool bResult = false;

        while (reader.Read())
        {
            int count = reader.GetInt32(0);
            if (count == 0)
            {
                bResult = false;
            }
            else if (count == 1)
            {
                bResult = true;
            } 
        }

        reader.Close();

        return bResult;
    }

    // Use this for initialization
    void Start ()
    {
       
    }
	
	// Update is called once per frame
	void Update ()
    {
		if (_AntiTimeoutTimer < Time.time && _conn != null)
        {
            //assume we're connected and can do this.  mysql and variants will disconnect after a few hours of inactivity by default, so this defeats that
            int junk = GetSecondsUntilMidnight(); //we don't really care, hopefully this won't be optimized out
            if (junk != -1) //just to rid of the warning
                print("SQL Heartbeat");
            _AntiTimeoutTimer = Time.time + _heartBeatTimerIntevalSeconds;
        }
	}


//note: You must do .Close() on what it returns (if not null)

    public MySqlDataReader Query(string sql)
    {
        try
        {
            MySqlCommand cmd = new MySqlCommand(sql, _conn);
            MySqlDataReader rdr = cmd.ExecuteReader();
            return rdr;
        }
        catch (MySqlException exx)
        {
            print("MySqlException: "+exx.Message);

            if (exx.Message=="Connection was killed")
            {
                DoEmergencyReconnect();
            }
        }
        catch (System.Exception exception)
        {
            // Error logging, post processing etc.
            print("System.Exception: " + exception.Message);

            if (exception.Message == "Connection must be valid and open.")
            {
                DoEmergencyReconnect();
            }

        }
        return null;
    }

    public int GetTableRecordCount(string tableName)
    {
        int result = 0;
        MySqlCommand cmd = new MySqlCommand("SELECT COUNT(*) from "+tableName, _conn);
        result = int.Parse(cmd.ExecuteScalar().ToString());
        return result;
    }

    public int QueryNoResult(string sql)
    {
        try
        {
            MySqlCommand cmd = new MySqlCommand(sql, _conn);
            return cmd.ExecuteNonQuery();
        }
        catch (MySqlException exx)
        {
            print("QueryNoResult "+exx.Message);
        }
        catch (System.Exception exception)
        {
            // Error logging, post processing etc.
            print("System.Exception: " + exception.Message);
        }

        return (int)SqlResult.ERROR; //error.  -1 is actually used to indicate certain things so we don't use that
    }

    public int GetSecondsUntilMidnight()
    {
        string sql = "SELECT TIMESTAMPDIFF(SECOND, NOW(), DATE(NOW()) +INTERVAL 1 DAY)";
        var result = Query(sql);

        result.Read();
        int seconds = (int)result.GetInt64(0);
        result.Close();

        /*
        for (int i = 0; i < result.FieldCount; i++)
        {
            print("Found a " + result.GetDataTypeName(i)+" name: "+result.GetName(i));

            print("Time: " + result.GetInt64(i));

                //" + " which is " + result.GetDateTime(i));
        }
        */

        return seconds;
    }

#endif
}
