using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using DG.Tweening;
using UnityEngine.SceneManagement;
using System.Runtime.InteropServices;

public static class StringExt
{
    public static string Truncate(this string value, int maxLength)
    {
        if (string.IsNullOrEmpty(value)) return value;
        return value.Length <= maxLength ? value : value.Substring(0, maxLength);
    }
}

public class RTUtil
{

    [DllImport("__Internal")]
    private static extern void JLIB_PopupUnblockSendMessage(string callbackObjectStr, string callbackFunctionStr, string callbackParmStr);

    [DllImport("__Internal")]
    private static extern void JLIB_PopupUnblockOpenURL(string URLStr);

    public static byte[] g_buffer = new byte[1024*50];
	public static int[] g_intArray = new int[3];
    public static uint[] g_uintArray = new uint[3];
    public static short[] g_shortArray = new short[3];
	public static float[] g_floatArray = new float[3];
    public static bool[] g_boolArray = new bool[3];

    //thanks to sol_hsa at http://sol.gfxile.net/interpolation/index.html
    //#define SMOOTHSTEP(x) ((x) * (x) * (3 - 2 * (x))) 

    /*
    
#define EASE_FROM(x) ((x)*(x))
#define SMOOTHSTEP_INVERSE(x) pow( ((x)/0.5)-1,3)
    */
    public static float SMOOTHSTEP(float x)
    {
        return (x) * (x) * (3 - 2 * (x));
    }

    public static float EASE_TO(float x)
    {
        return (1 - (1 - (x)) * (1 - (x)));
    }


    private static StringBuilder AppendColor(StringBuilder sb, int r, int g, int b, int alpha)
    {
        sb.Append("<color=#" + r.ToString("x2") + g.ToString("x2") + b.ToString("x2") + alpha.ToString("x2") + ">");
        //sb.Append("<color=#0000ffff>");
        
        return sb;
    }

	public static float RoundToOneDec(float f)
	{
		return Mathf.Round(f * 10) / 10;
	}
	public static float RoundToTwoDec(float f)
	{
		return Mathf.Round(f * 100) / 100;
	}

	public static string ConvertSansiToUnityColors(string s)
    {
        System.Text.StringBuilder sb = new System.Text.StringBuilder("");
        int colorDepth = 0;

        for (int i = 0; i < s.Length; i++)
        {

            if (s[i] == '{')
            {
                //found sansi control code

                int closingIndex = s.IndexOf('}', i);
                if (closingIndex == -1)
                {
                    RTConsole.Log("Malformed SANSI detected in " + s);
                    continue;
                }

                //a 0 length link will screw up if it's at the ending and beginning of a line.. using a zero width unicode space fixes it without
                //actually adding a character.  I hope
                sb.Append("<link=\"" + s.Substring(i + 1, (closingIndex - i)-1)+"\">"+System.Convert.ToChar(0x200b)+"</link>");
                i = closingIndex; //move past
                continue;
            }

            if (s[i] == '`')
            {
                //found special code
                if (s.Length-1 > i)
                {
                    i++; //move it past the second control code
              
                    if (s[i] == '`')
                    {
                        if (colorDepth > 0)
                        {
                            sb.Append("</color>");
                            colorDepth--;
                        }
                        continue;
                    }

                   colorDepth++;
                    //enough room for a code here
                    if (s[i] == '0')
                    {
                        AppendColor(sb, 100, 255, 0, 255);
                        continue;
                    }
                    if (s[i] == '1')
                    {
                        AppendColor(sb, 173, 244, 255, 255);
                        continue;
                    }
                    if (s[i] == '2')
                    {
                        AppendColor(sb, 50, 160, 0, 255);
                        continue;
                    }
                    if (s[i] == '3')
                    {
                        AppendColor(sb, 191, 218, 255, 255);
                        continue;
                    }
                    if (s[i] == '4')
                    {
                        AppendColor(sb, 255, 39, 29, 255);
                        continue;
                    }
                    if (s[i] == '5')
                    {
                        AppendColor(sb, 235, 183, 255, 255);
                        continue;
                    }
                    if (s[i] == '6')
                    {
                        AppendColor(sb, 255, 202, 111, 255);
                        continue;
                    }
                    if (s[i] == '7')
                    {
                        AppendColor(sb, 230, 230, 230, 255);
                        continue;
                    }
                    if (s[i] == '8')
                    {
                        AppendColor(sb, 255, 148, 69, 255);
                        continue;
                    }
                    if (s[i] == '9')
                    {
                        AppendColor(sb, 255, 238, 12, 255);
                        continue;
                    }
                    if (s[i] == '!')
                    {
                        AppendColor(sb, 209, 255, 249, 255);
                        continue;
                    }
                    if (s[i] == '@')
                    {
                        AppendColor(sb, 255, 205, 201, 255);
                        continue;
                    }
                    if (s[i] == '#')
                    {
                        AppendColor(sb, 255, 143, 243, 255);
                        continue;
                    }
                    if (s[i] == '$')
                    {
                        AppendColor(sb, 255, 252, 197, 255);
                        continue;
                    }
                    if (s[i] == '^')
                    {
                        AppendColor(sb, 181, 255, 151, 255);
                        continue;
                    }
                    if (s[i] == '&')
                    {
                        AppendColor(sb, 254, 235, 255, 255);
                        continue;
                    }
                    if (s[i] == 'w')
                    {
                        AppendColor(sb, 255, 255, 255, 255);
                        continue;
                    }
                    if (s[i] == 'o')
                    {
                        AppendColor(sb, 252, 230, 186, 255);
                        continue;
                    }
                    if (s[i] == 'b')
                    {
                        AppendColor(sb, 0, 0, 0, 255);
                        continue;
                    }
                    if (s[i] == 'p')
                    {
                        AppendColor(sb, 255, 223, 241, 255);
                        continue;
                    }
                  


                    //uh oh, unknown color code
                    colorDepth--; //Remove the one we had added earlier
                    continue;
                }
            }
            sb.Append(s[i]);
        }

        while (colorDepth > 0)
        {
            colorDepth--;
            sb.Append("</color>");
        }
        return sb.ToString();
    }

    public static string NameFilter(string str)
    {
        char[] arr = str.Where(c => (char.IsLetterOrDigit(c))).ToArray();
        return new string(arr);
    }

    public static string ChatFilter(string str)
    {
        char[] arr = str.Where(c => (char.IsLetterOrDigit(c) ||
        char.IsPunctuation(c) ||
        char.IsWhiteSpace(c)
        || c == '`')).ToArray();
        return new string(arr);
    }

    //write
    public static void SerializeString(string value, byte[] byteBuff, ref int index)
	{

		//first write the size of it
		SerializeInt32(value.Length, byteBuff, ref index); 

		//now write the actual string data
		//OPTIMIZE - couldn't we do this without allocating new data here?
		byte[] stringBuff = Encoding.ASCII.GetBytes(value);

		System.Buffer.BlockCopy(stringBuff, 0, byteBuff, index, value.Length); 
		index += value.Length;
	}

	//read
	public static bool SerializeString(ref string value, byte[] byteBuff, ref int index)
	{
	//first get the size
		int tempInt = 0;

        try
        {
            SerializeInt32(ref tempInt, byteBuff, ref index);

            value = System.Text.Encoding.Default.GetString(byteBuff, index, tempInt);

            index += tempInt;
        }
        catch (System.Exception e)
        {
            
            RTConsole.Log("Config: " + e.Message);
            return false; //signal error
        }

        return true;
	}

	//write
	public static void SerializeInt32(int value, byte[] byteBuff, ref int index)
	{
		g_intArray[0] = value;
		System.Buffer.BlockCopy(g_intArray, 0, byteBuff, index, 4); //raw copy over 4 bytes
		index += 4;
	}

	//read
	public static void SerializeInt32(ref int value, byte[] byteBuff, ref int index)
	{
		System.Buffer.BlockCopy(byteBuff, index, g_intArray, 0, 4); //raw copy over 4 bytes
		value = g_intArray[0];
		index += 4;
	}

    //write
    public static void SerializeUInt32(uint value, byte[] byteBuff, ref int index)
    {
        g_uintArray[0] = value;
        System.Buffer.BlockCopy(g_uintArray, 0, byteBuff, index, 4); //raw copy over 4 bytes
        index += 4;
    }

    //read
    public static void SerializeUInt32(ref uint value, byte[] byteBuff, ref int index)
    {
        System.Buffer.BlockCopy(byteBuff, index, g_uintArray, 0, 4); //raw copy over 4 bytes
        value = g_uintArray[0];
        index += 4;
    }

    //write
    public static void SerializeBool(bool value, byte[] byteBuff, ref int index)
    {
        g_boolArray[0] = value;
        System.Buffer.BlockCopy(g_boolArray, 0, byteBuff, index, 1); //raw copy over 4 bytes
        index += 1;
    }

    //read
    public static void SerializeBool(ref bool value, byte[] byteBuff, ref int index)
    {
        System.Buffer.BlockCopy(byteBuff, index, g_boolArray, 0, 1); //raw copy over 4 bytes
        value = g_boolArray[0];
        index += 1;
    }

    //write
    public static void SerializeInt16(short value, byte[] byteBuff, ref int index)
	{
		g_shortArray[0] = value;
		System.Buffer.BlockCopy(g_shortArray, 0, byteBuff, index, 2); //raw copy over 4 bytes
		index += 2;
	}
	
	//read
	public static void SerializeInt16(ref short value, byte[] byteBuff, ref int index)
	{
		System.Buffer.BlockCopy(byteBuff, index, g_shortArray, 0, 2); //raw copy over 4 bytes
		value = g_shortArray[0];
		index += 2;
	}

	//write
	public static void SerializeFloat(float value, byte[] byteBuff, ref int index)
	{
		g_floatArray[0] = value;
		System.Buffer.BlockCopy(g_floatArray, 0, byteBuff, index, 4); //raw copy over 4 bytes
		index += 4;
	}
	
	//read
	public static void SerializeFloat(ref float value, byte[] byteBuff, ref int index)
	{
		System.Buffer.BlockCopy(byteBuff, index, g_floatArray, 0, 4); //raw copy over 4 bytes
		value = g_floatArray[0];
		index += 4;
	}

	//write
	public static void SerializeVector2(Vector2 value, byte[] byteBuff, ref int index)
	{
		g_floatArray[0] = value.x;
		g_floatArray[1] = value.y;
		System.Buffer.BlockCopy(g_floatArray, 0, byteBuff, index, 8); 
		index += 8;
	}
	
	//read
	public static void SerializeVector2(ref Vector2 value, byte[] byteBuff, ref int index)
	{
		System.Buffer.BlockCopy(byteBuff, index, g_floatArray, 0, 8); 
		value.x = g_floatArray[0];
		value.y = g_floatArray[1];
		index += 8;
	}

    //write
    public static void SerializeVector2Int(Vector2Int value, byte[] byteBuff, ref int index)
    {
        g_intArray[0] = value.x;
        g_intArray[1] = value.y;
        System.Buffer.BlockCopy(g_intArray, 0, byteBuff, index, 8);
        index += 8;
    }

    //read
    public static void SerializeVector2Int(ref Vector2Int value, byte[] byteBuff, ref int index)
    {
        System.Buffer.BlockCopy(byteBuff, index, g_intArray, 0, 8);
        value.x = g_intArray[0];
        value.y = g_intArray[1];
        index += 8;
    }

    public static Vector2 Vector2Lerp(Vector2 from, Vector2 to, float t)
	{
		return new Vector2(from.x - ((from.x - to.x) * t), from.y - ((from.y - to.y) * t));
	}


    public static float SinGamePulseBySeconds(float intervalSeconds)
    {
        return Mathf.Sin(  (Time.time/ (intervalSeconds/2)));
        //return Mathf.Sin(Time.time);
    }

    //a helper to use with the above functions when you just want 0-1 range
    public static float SinToZeroToOneRange(float sinIn)
    {
        return (sinIn + 1.0f) / 2;
    }


    static public string SetFirstCharacterToUpperCase(string s)
    {
        if (string.IsNullOrEmpty(s))
        {
            return string.Empty;
        }
        char[] a = s.ToCharArray();
        a[0] = char.ToUpper(a[0]);
        return new string(a);
    }

    static public string GetRandomName()
    {
        int i = Random.Range(0, 10+1);
        string r = "";

        switch (i)
        {
            case 0: r = "gar"; break;
            case 1: r = "num"; break;
            case 2: r = "gren"; break;
            case 3: r = "dar"; break;
            case 4: r = "fal"; break;
            case 5: r = "kor"; break;
            case 6: r = "bol"; break;
            case 7: r = "dor"; break;
            case 8: r = "ken"; break;
            case 9: r = "ren"; break;
            case 10: r = "zar"; break;

            default:
                r = "Crap";
                break;
        }
        return r;
    }

    //gets whatever is after the parm you sent in.  Ie, "-name" would return "seth" if the command line was "-name seth"
    public static string GetCommandLineSetting(string name)
    {
        var line = System.Environment.GetCommandLineArgs();
        for (int i = 0; i < line.Length; i++)
        {
            if (line[i] == name && line.Length > i + 1)
            {
                return line[i + 1];
            }
        }
        return null;
    }

    //checks for the existence of a specific word in the command line
    public static bool DoesCommandLineWordExist(string name)
    {
        var line = System.Environment.GetCommandLineArgs();
        for (int i = 0; i < line.Length; i++)
        {
            if (line[i] == name)
            {
                return true;
            }
        }
        return false;
    }

    static public bool SetActiveByNameIfExists(string name, bool bActive)
    {
        GameObject obj = GameObject.Find(name);
        if (obj != null)
        {
            obj.SetActive(bActive);
            return true; //found it
        }
        return false; //couldn't find it
    }

    //hideously slow as it iterates all objects, so don't overuse!
    public static GameObject FindInChildrenIncludingInactive(GameObject go, string name)
    {

        for (int i=0; i < go.transform.childCount; i++)
        {
            if (go.transform.GetChild(i).gameObject.name == name) return go.transform.GetChild(i).gameObject;
            GameObject found = FindInChildrenIncludingInactive(go.transform.GetChild(i).gameObject, name);
            if (found != null) return found;
        }

        return null;  //couldn't find crap
    }
    
    //hideously slow as it iterates all objects, so don't overuse!
    public static GameObject FindIncludingInactive(string name)
    {
        Scene scene = SceneManager.GetActiveScene();
        var game_objects = new List<GameObject>();
        scene.GetRootGameObjects(game_objects);

        foreach (GameObject obj in game_objects)
        {
            if (obj.transform.name == name) return obj;

            GameObject found = FindInChildrenIncludingInactive(obj, name);
            if (found) return found;
         }

        return null;
    }

    public static GameObject FindObjectOrCreate(string name)
    {
        GameObject temp = GameObject.Find(name);
        if (temp ==  null)
        {
            //create it
            temp = new GameObject();
            temp.name = name;
        }

        return temp;
    }

    //Below snippet taken from https://forum.unity.com/threads/deleting-all-chidlren-of-an-object.92827/, credit to mwk888
    /// <summary>
    /// Calls GameObject.Destroy on all children of transform. and immediately detaches the children
    /// from transform so after this call tranform.childCount is zero.
    /// </summary>
    public static void DestroyChildren(Transform transform)
    {
        for (int i = transform.childCount - 1; i >= 0; --i)
        {
            GameObject.Destroy(transform.GetChild(i).gameObject);
        }
        transform.DetachChildren();
    }

    //Note: This will also activate/inactivate the object
    static public void FadeGUIPanelByName(string name, float alphaTarget, float duration = 1.0f, float delayBeforeStart = 0.0f)
    {

        GameObject panelObj = FindIncludingInactive(name);
        
        if (panelObj == null)
        {
            RTConsole.Log("Can't find panel by the name of " + name + ", aborting FadeGUIPanelByName");
            return;
        }

        if (panelObj == null)
        {
            Debug.Log("FadeGUIPanelByName: Can't find " + name);
            return;
        }

        if (!panelObj.activeInHierarchy)
        {
            panelObj.SetActive(true); //can't fade anything if it's off!
        }
     

        CanvasGroup canvasGroup = panelObj.GetComponent<CanvasGroup>();

        if (canvasGroup == null)
        {
          //  Debug.Log("Adding canvas group, wonder if this works..");
            canvasGroup = panelObj.AddComponent<CanvasGroup>();
        }

        /*
        TweenTracker tracker = panelObj.GetComponent<TweenTracker>();
        if (tracker == null)
        {
            //  Debug.Log("Adding canvas group, wonder if this works..");
            tracker = panelObj.AddComponent<TweenTracker>();
        }

    */

        DOTween.Kill(panelObj); //kill any outstanding commands...
        Sequence mySequence = DOTween.Sequence();
        mySequence.SetId(panelObj);

        mySequence.AppendInterval(delayBeforeStart);
        //RTConsole.Log("Setting panel " + panelObj.ToString() + " to alpha " + alphaTarget + " in " + duration);

        mySequence.Append(canvasGroup.DOFade(alphaTarget, duration));

        //canvasGroup.interactable = false; //stop additional clicking
        if (alphaTarget == 0)
        {
            canvasGroup.blocksRaycasts = false;
        } else
        {
            canvasGroup.blocksRaycasts = true;
        }

        //let's also inactivate it so it won't waste cycles
        mySequence.OnComplete(() =>
           {

               if (alphaTarget == 0)
               {
                  panelObj.SetActive(false);
               } 
              
             // canvasGroup.interactable = true; //turn this back on
               canvasGroup.blocksRaycasts = true;
           });
       

        mySequence.SetAutoKill(true);
        mySequence.Play();
      
    }

    //Note: This will also activate/inactivate the object
    static public void FadeGUIImageByName(string name, float alphaTarget, float duration = 1.0f, float delayBeforeStart = 0.0f)
    {

        GameObject image = FindIncludingInactive(name);

        if (image == null)
        {
            RTConsole.Log("Can't find Image by the name of " + name + ", aborting FadeGUIImageByName");
            return;
        }

        if (image == null)
        {
            Debug.Log("FadeGUIImageByName: Can't find " + name);
            return;
        }

        if (!image.activeInHierarchy)
        {
            image.SetActive(true); //can't fade anything if it's off!
        }


        UnityEngine.UI.Image imageComp = image.GetComponent<UnityEngine.UI.Image>();

        if (imageComp == null)
        {
            //  Debug.Log("Adding canvas group, wonder if this works..");
            RTConsole.Log("Error, can't find image in object named " + name);
            return;
        }

        /*
        TweenTracker tracker = image.GetComponent<TweenTracker>();
        if (tracker == null)
        {
            //  Debug.Log("Adding canvas group, wonder if this works..");
            tracker = image.AddComponent<TweenTracker>();
        }

    */

        DOTween.Kill(image); //kill any outstanding commands...
        Sequence mySequence = DOTween.Sequence();
        mySequence.SetId(image);

        mySequence.AppendInterval(delayBeforeStart);
        //RTConsole.Log("Setting Image " + image.ToString() + " to alpha " + alphaTarget + " in " + duration);

        mySequence.Append(imageComp.DOFade(alphaTarget, duration));

        //canvasGroup.interactable = false; //stop additional clicking

/*
        if (alphaTarget == 0)
        {
            imageComp.raycastTarget
        }
        else
        {
            imageComp.blocksRaycasts = true;
        }
        */

        //let's also inactivate it so it won't waste cycles
        mySequence.OnComplete(() =>
        {

            if (alphaTarget == 0)
            {
                image.SetActive(false);
            }

            // canvasGroup.interactable = true; //turn this back on
           // imageComp.blocksRaycasts = true;
        });

        mySequence.SetAutoKill(true);
        mySequence.Play();

    }

    //Note: This will also activate/inactivate the object
    static public void SetFadeOfPanelByName(string name, float alphaTarget)
    {

        GameObject panelObj = FindIncludingInactive(name);

        if (panelObj == null)
        {
            RTConsole.Log("Can't find panel by the name of " + name + ", aborting SetFadeOfPanelByName");
            return;
        }
       
        if (!panelObj.activeInHierarchy)
        {
            panelObj.SetActive(true); //can't fade anything if it's off!
        }
       
        if (panelObj == null)
        {
            Debug.Log("SetFadeOfPanelByName: Can't find " + name);
            return;
        }

        CanvasGroup canvasGroup = panelObj.GetComponent<CanvasGroup>();

        if (canvasGroup == null)
        {
            //  Debug.Log("Adding canvas group, wonder if this works..");
            canvasGroup = panelObj.AddComponent<CanvasGroup>();
        }

        canvasGroup.alpha = alphaTarget;

        if (alphaTarget == 1)
        {

            if (!panelObj.activeInHierarchy)
            {
                panelObj.SetActive(false); //can't fade anything if it's off!
            }
        }

    }
    //Note: On webgl builds, this will allow things that create new browser windows (a facebook logon, for example) to work properly.  However, it MUST
    //be called from a callback from an OnPointerDown().  Use RTButton instead of Button for this.

    static public void PopupUnblockSendMessage(string objectName, string functionName, string optionalParm)
    {
#if UNITY_WEBGL && !UNITY_EDITOR
        JLIB_PopupUnblockSendMessage(objectName, functionName, optionalParm);
#else
        //just do it the normal way, it's not needed
        GameObject o = GameObject.Find(objectName);

        if (!o)
        {
            Debug.Log("PopupUnblockSendMessage: Can't find gameobject " + objectName);
            return;
        }

        o.SendMessage(functionName, optionalParm);
#endif
    }

    static public void PopupUnblockOpenURL(string URL)
    {
#if UNITY_WEBGL && !UNITY_EDITOR
        JLIB_PopupUnblockOpenURL(URL);
#else
        Application.OpenURL(URL);
#endif
    }

    static public void AccelerateTo(Rigidbody body, Vector3 targetVelocity, float maxAccel)
    {
        Vector3 deltaV = targetVelocity - body.velocity;
        Vector3 accel = deltaV;

        if (accel.sqrMagnitude > maxAccel * maxAccel)
            accel = accel.normalized * maxAccel;

        body.AddForce(accel, ForceMode.Acceleration);
    }

    static public void SetButtonText(string buttonName, string newText)
    {
        GameObject buttonObj = RTUtil.FindIncludingInactive(buttonName);

        if (!buttonObj)
        {
            RTConsole.Log("Can't find button named " + buttonName);
            return;
        }

        //RTConsole.Log("Setting button to " + newText);

#if !RT_NO_TEXMESH_PRO
        TMPro.TextMeshProUGUI textMeshPro = buttonObj.transform.GetComponentInChildren<TMPro.TextMeshProUGUI>();
        textMeshPro.text = newText;
#else
        RTConsole.Log("Add support for normal UGUI button text changing here");
#endif
    }

    

}
