/*

Source code by Seth A. Robinson

 */

//#define RT_NOAUDIO
 
using UnityEngine;
using System.Collections;
using System;
using System.IO;
using System.Text;
using UnityEngine.UI;
using System.Collections.Generic;
using DG.Tweening;
using TMPro;

//using UnityEngine.Networking;

public enum eGameMode
{
    START_MENU,
    PLAYING,
    DEAD,
    END_MENU,
    END_MENU_DONE
}

public class GameLogic : MonoBehaviour
{
    static GameLogic _this = null;
    public GameObject _player;
    public List<AudioClip> _clips;
    public GameObject _sweeperPrefab;
    public GameObject _barrierLeftPrefab;
    Spawner _spawner;
    public TextMeshProUGUI _scoreDisplay;
    public GameObject _gravePrefab;

    int _score;

    public eGameMode _gameMode = eGameMode.START_MENU;


    public static string GetName()
    {
        return Get().name;
    }

    private void Awake()
    {
        _spawner = GetComponent<Spawner>();

        Debug.Log("Turning gravity off");

        Physics2D.gravity = new Vector2(0, 0);

        Application.targetFrameRate = 100;
        QualitySettings.vSyncCount = 0;
        //QualitySettings.antiAliasing = 4;
        /*
        Debug.unityLogger.filterLogType = LogType.Log;

        Application.SetStackTraceLogType(LogType.Log, StackTraceLogType.ScriptOnly);
        Application.SetStackTraceLogType(LogType.Error, StackTraceLogType.ScriptOnly);
        Application.SetStackTraceLogType(LogType.Assert, StackTraceLogType.ScriptOnly);
        Application.SetStackTraceLogType(LogType.Exception, StackTraceLogType.ScriptOnly);
        Application.SetStackTraceLogType(LogType.Warning, StackTraceLogType.ScriptOnly);
        */

      
    }

    // Use this for initialization
    void Start()
    {

        RTAudioManager.Get().AddClipsToLibrary(_clips);

        DOTween.Init(true, true, LogBehaviour.Verbose).SetCapacity(200, 20);
        // RTAudioManager.Get().SetDefaultMusicVol(0.4f);
        _this = this;

#if RT_NOAUDIO
		AudioListener.pause = true;
#endif

     RTConsole.Get().SetShowUnityDebugLogInConsole(true);
       
        string version = "Unity V "+ Application.unityVersion+" :";

        #if RT_BETA
        print ("Beta build detected!");
#endif

        RTConsole.Get().SetMirrorToDebugLog(true);


        RTMessageManager.Get().Schedule(0.1f, CreateMainMenu); //avoid conflict with GUIHide's Start from MainMenu
        //CreateEndMenu();
    }

    static public GameLogic Get()
	{
		return _this;
	}
 
	void OnApplicationQuit() 
	{
        // Make sure prefs are saved before quitting.
        //PlayerPrefs.Save();
        RTConsole.Log("Application quitting normally");

//        NetworkTransport.Shutdown();
        print("QUITTING!");
    }
    

    private void OnDestroy()
    {
        print("Game logic destroyed");
    }

    public void OnScoreMod(int mod)
    {
        _score += mod;
        _scoreDisplay.text = _score.ToString();
    }

    public void CreateMainMenu()
    {

        RTAudioManager.Get().PlayMusic("SummerOfSynth_CH_trimmed", 0.8f, 1.0f, false);
      //  RTUtil.FadeGUIPanelByName("GameGUI", 0);
        Destroy(RTUtil.FindIncludingInactive("TempCrap"));

        Destroy(RTUtil.FindIncludingInactive("BarrierLeft"));
        Destroy(RTUtil.FindIncludingInactive("ObjectSweeper"));
        Debug.Log("Creating main menu");
        SetGameMode(eGameMode.START_MENU);


        var sweep = Instantiate(_sweeperPrefab);
        sweep.name = "ObjectSweeper";

        var bar = Instantiate(_barrierLeftPrefab);
        bar.name = "BarrierLeft";

        RTUtil.FadeGUIPanelByName("MainMenu", 1, 0.2f, 0.1f);

    }

    public void CreateEndMenu()
    {
        RTUtil.FadeGUIPanelByName("GameGUI", 0, 5, 1);

        RTUtil.FadeGUIPanelByName("EndMenu", 1, 2, 0.1f);
        SetGameMode(eGameMode.END_MENU);

       // _score = 20;

        var parentRoot = RTUtil.FindIncludingInactive("EndMenu");

        var parentGUI = new GameObject("TempCrap");
        parentGUI.transform.parent = parentRoot.transform;
        parentGUI.transform.localPosition = new Vector3(0, 0, 0);
        float x = -350;
        float y = 256;
        float z = 0;

        var pos = new Vector3(x, y, z);
        RTMessageManager.Get().Schedule(0, RTAudioManager.Get().PlayMusic, "JoshuaTreeWindstorm_CH_PartA");

        float time = 2;

        for (int i=0; i < _score; i++)
        {
            var ob = Instantiate(_gravePrefab, pos, Quaternion.identity, parentGUI.transform);
            ob.transform.localPosition = pos;
            pos.x += 50;

            RTMessageManager.Get().Schedule(time, RTAudioManager.Get().Play, "pong_wall_bump0");
            ob.transform.DOMoveY(-300, 1, true).From().SetDelay(time);
            
            time += 0.1f;

                if (pos.x > 350)
            {
                pos.x = x;
                pos.y -= 20;
                pos.z += 0.1f;

                if (pos.y < -280)
                {
                    pos.y = 250;
                }
            }
        }
     
        RTMessageManager.Get().Schedule(time, SetGameMode, eGameMode.END_MENU_DONE);

    }

    public void KillMainMenu()
    {
        RTUtil.FadeGUIPanelByName("MainMenu", 0, 0.5f, 0.0f);
    }

    public void StartGame()
    {
        var go = RTUtil.FindIncludingInactive("Level");

        if (go != null)
        {
            go.name = "LevelOld";
            Destroy(go);
        }

        //spawn main guy

        var player = RTUtil.FindIncludingInactive("Player");
        if (player != null)
        {
            Destroy(player);
        }


        GameObject cam = GameObject.Find("Camera");
        var camScript = cam.GetComponent<CameraLogic>();

        camScript.InitForNewGame();

        var p = Instantiate(_player, camScript.GetPlayerStartPosition(), Quaternion.identity, null);
        Node nscript = p.GetComponent<Node>();
        nscript.SetControlMode(eControlMode.PLAYER);

        ResetGame();
        RTAudioManager.Get().PlayMusic("JoshuaTreeWindstorm_CH_PartB", 0.8f, 1.0f, false);
    }

    void ResetGame()
    {
        _score = 0;
        RTUtil.FadeGUIPanelByName("GameGUI", 1, 2, 15);
        OnScoreMod(0);
        _spawner.Reset();
    }

    public void SetGameMode(eGameMode mode)
    {
        _gameMode = mode;
        //Debug.Log("Game mode changed to " + mode);

        switch(mode)
        {
            case eGameMode.DEAD:

                //player lost the whole game
                RTMessageManager.Get().Schedule(0, RTAudioManager.Get().StopMusic);
               // RTMessageManager.Get().Schedule(1, RTAudioManager.Get().PlayEx, "blips_lose2", 1.0f, 1.0f, false, 0.0f);
                RTMessageManager.Get().Schedule(0.5f, GameLogic.Get().CreateEndMenu);
                break;
        }
    }
    // Update is called once per frame

    void Update () 
    {
        switch (_gameMode)
        {

            case eGameMode.START_MENU:
                {
                    bool bButtonDown = false;
                    bButtonDown = (Input.GetButtonDown("Fire1") || Input.GetKeyDown(KeyCode.Space) || Input.GetKeyDown(KeyCode.LeftControl));

                    if (bButtonDown)
                    {
                        KillMainMenu();
                        StartGame();
                        _gameMode = eGameMode.PLAYING;
                    }
                }
                break;

            case eGameMode.END_MENU_DONE:
                {
                    bool bButtonDown = false;
                    bButtonDown = (Input.GetButtonDown("Fire1") || Input.GetKeyDown(KeyCode.Space) || Input.GetKeyDown(KeyCode.LeftControl));

                    if (bButtonDown)
                    {
                        RTUtil.FadeGUIPanelByName("EndMenu", 0, 1, 0);
                        CreateMainMenu();
                    }
                }
                break;
        }
    }

}
