﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


public enum eControlMode
{
    PLAYER,
    BOT

}

public class Node : MonoBehaviour
{
    public GameObject _playerBullet;
    Rigidbody2D _rigid;
    GameObject _parent;
    float _fireCooldownSeconds = 0.4f;
    float _fireCoolDown;
    bool _isDead = false;
    GameObject _barrierLeft;

    public Material _man2;
    public Material _man3;

    List<GameObject> _children = new List<GameObject>();

    public eControlMode _controlMode = eControlMode.BOT;

    private void Awake()
    {
        _rigid = GetComponent<Rigidbody2D>();
    }

    // Start is called before the first frame update

    public void SetControlMode(eControlMode mode)
    {
        _controlMode = mode;

        if (_controlMode == eControlMode.BOT)
        {
            GetComponentInChildren<MeshRenderer>().material.color = new Color(1, 1, 1, 0.5f);
        } else
        {
            gameObject.layer = LayerMask.NameToLayer("AttachedNode");
            GetComponentInChildren<MeshRenderer>().material.color = new Color(1, 1, 1, 1);

        }
    }

    void Start()
    {
        SetControlMode(_controlMode);
    }

    void PlayerFixedUpdate()
    {
        var x = Input.GetAxis("Horizontal") * Time.deltaTime;
        var z = Input.GetAxis("Vertical") * Time.deltaTime;

        float speed = 3;

        var pos = _rigid.position;
        pos.x += x * speed;
        pos.x += CameraLogic.Get().GetFixedSpeed();

        pos.y += z * speed;

        _rigid.MovePosition(pos);


    }

    void Connect(GameObject child, GameObject parent)
    {

        if (child.GetComponent<FixedJoint2D>() != null)
        {
            Debug.Log("Warning, " + child.name + " already has a joint2d");
            return;
        }

        var newJoint = child.AddComponent<FixedJoint2D>();
        newJoint.connectedBody = parent.GetComponent<Rigidbody2D>();
        child.layer = LayerMask.NameToLayer("AttachedNode");
        _parent = parent;
        //newJoint.dampingRatio = 0.1f;
        newJoint.breakForce = 300;
        newJoint.breakTorque = 300;

        RTAudioManager.Get().PlayEx("speedup_start", 1, 1, false, 0.1f);
        parent.GetComponent<Node>().AddChild(gameObject);
    }

    public void AddChild(GameObject child)
    {
        _children.Add(child);
    }

    private void OnJointBreak2D(Joint2D joint)
    {
        if (joint == GetComponent<Joint2D>())
        {
            OnDie(true, true);
        }
    }

    void SetMaterial(Material mat)
    {
        var mr = gameObject.GetComponentInChildren<MeshRenderer>();
        mr.material = mat;
    }

    public void OnDie(bool giveScore, bool doFancyDeath)
    {
        if (_isDead)
        {
            return;
        }

        _isDead = true;

        float timeToDie = 0.2f;
        float inc = 0.2f;

        foreach (GameObject go in _children)
        {
            if (go != null)
            {
                Node nscript = go.GetComponent<Node>();

                if (nscript != null)
                {
                    RTMessageManager.Get().Schedule(timeToDie, nscript.OnDie, true, true);
                   
                    timeToDie += inc;
                }
            }
           
        }

        _children.Clear();
        RTAudioManager.Get().PlayEx("speedup_end", 1, 1, false, 0.1f);
        if (giveScore)
            GameLogic.Get().OnScoreMod(1);

        if (doFancyDeath)
        {

            var bc = gameObject.GetComponent<BoxCollider2D>();

            if (bc != null)
            {
                bc.enabled = false;
            }
            

            SetMaterial(_man2);
            RTMessageManager.Get().Schedule(0.3f, SetMaterial, _man3);
            Destroy(gameObject, 0.6f);
        }
        else
        {
            Destroy(gameObject);

        }

        if (_controlMode == eControlMode.PLAYER)
        {
            GameLogic.Get().SetGameMode(eGameMode.DEAD);
        }

    }
    public void Suicide()
    {
        if (CanBeAParent() && _controlMode != eControlMode.PLAYER)
        {
            OnDie(true, true);
        }
    }
    public bool CanBeAParent()
    {
        if (_controlMode == eControlMode.PLAYER) return true;
        if (_parent != null) return true;
        return false;
    }

    public void OnSpiked()
    {
        OnDie(true, false);
    }


    private void OnTriggerEnter2D(Collider2D collision)
    {
        if (_controlMode == eControlMode.BOT && _parent == null)
        {
            if (collision.gameObject.CompareTag("PlayerBullet"))
            {
                Destroy(collision.gameObject);
                OnDie(true, true);
            }
        }
    }


    private void OnCollisionEnter2D(Collision2D collision)
    {

        var nodeScript = collision.gameObject.GetComponent<Node>();
        if (nodeScript == null)
        {

                return; //don't care
        }

        //Debug.Log(gameObject.name + " hit by " + collision.gameObject.name);

        if (_controlMode == eControlMode.BOT && _parent == null)
        {
            if (!nodeScript.CanBeAParent()) return;
            Connect(gameObject, collision.gameObject);
        } else
        {

            //Connect(collision.gameObject, gameObject);
        }

    }
// 
//     private void OnCollisionStay2D(Collision2D collision)
//     {
//         Debug.Log(gameObject.name + " crushed with " + collision.GetContact(0).tangentImpulse);
// 
//         if (Mathf.Abs(collision.GetContact(0).tangentImpulse) > 1.4f)
//         {
//              OnDie(); 
//         }
// 
//     }

    void CheckForFire()
    {
        

        if (_fireCoolDown > Time.time) return;

        if (_parent == null && _controlMode != eControlMode.PLAYER) return;

        var f = Mathf.Max( Input.GetAxis("Jump"), Input.GetAxis("Fire1"));

       // Debug.Log(f);
       if (f >0)
        {
            _fireCoolDown = Time.time+_fireCooldownSeconds;
            Instantiate(_playerBullet, transform.position, Quaternion.identity, null);
            RTAudioManager.Get().PlayEx("gun", 1, 3, false, 0.1f);


        }

    }

    void CheckForSquish()
    {
        if (_parent == null && _controlMode == eControlMode.BOT) return;

      
        if (_barrierLeft == null)
        {
            _barrierLeft = GameObject.Find("BarrierLeft");

        }
        //var barCollider = bar.GetComponent<BoxCollider2D>();
        if (_barrierLeft.transform.position.x+0.421f > transform.position.x)
        {
            RTAudioManager.Get().Play("squish");
            OnDie(true, true); //crushed
        }
    }
    // Update is called once per frame
    void FixedUpdate()
    {
        if (_isDead) return;

        switch(_controlMode)
        {
            case eControlMode.PLAYER:

                if (!_isDead)
                {
                    PlayerFixedUpdate();
                   // CheckForFire();
                    CheckForSquish();
                }
                break;

            case eControlMode.BOT:
                CheckForFire();
                CheckForSquish();

                break;
        }
      


    }
}
