﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Spawner : MonoBehaviour
{

    int _curSector = 0;
    int _lastSpawnedSector = 0;
    public GameObject _wall;
    public GameObject _node;
    public GameObject _spike;
    public GameObject _tracker;



    // Start is called before the first frame update
    void Start()
    {
        Reset();
    }

    public void Reset()
    {
        _curSector = 0;
        _lastSpawnedSector = 0;
    }

    Vector3 SectorToPos(float sector, float x, float y)
    {
        sector *= CameraLogic.C_SECTOR_SIZE_X;

        return new Vector3(sector + x, y - (CameraLogic.C_SECTOR_SIZE_Y / 2)+1);
    }

    void SpawnRandom(int sector, float wallPercent = 0.33f, float peoplePercent = 0.33f, float trapPercent = 0.33f)
    {

        for (int y = 0; y < CameraLogic.C_SECTOR_SIZE_Y; y++)
        {
            for (int x = 0; x < CameraLogic.C_SECTOR_SIZE_X; x++)
            {

                Vector3 pos = SectorToPos(sector, x - 0.5f, y - 0.5f);

                float r = Random.value;



                if (r < wallPercent)
                {
                    pos.z += 1;

                    var obj = Instantiate(_wall, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);
                    continue;
                }

                r = Random.value;

                 if (r < peoplePercent)
                {
                    var obj = Instantiate(_node, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);
                    continue;
                }

                r = Random.value;
                if (r < trapPercent)
                {
                    var obj = Instantiate(_spike, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);
                }

            }
        }

    }


    void SpawnRandomSmall(int sector, float wallPercent = 0.33f, float peoplePercent = 0.33f, float trapPercent = 0.33f)
    {

        for (int y = 0; y < CameraLogic.C_SECTOR_SIZE_Y; y++)
        {
            for (int x = 0; x < CameraLogic.C_SECTOR_SIZE_X/2; x++)
            {

                Vector3 pos = SectorToPos(sector, x - 0.5f, y - 0.5f);

                float r = Random.value;



                if (r < wallPercent)
                {
                    pos.z += 1;

                    var obj = Instantiate(_wall, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);
                    continue;
                }

                r = Random.value;

                if (r < peoplePercent)
                {
                    var obj = Instantiate(_node, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);
                    continue;
                }

                r = Random.value;
                if (r < trapPercent)
                {
                    var obj = Instantiate(_spike, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);
                }

            }
        }

    }

    void SpawnVerticalStripWithHole(int sector, int x)
    {

        int numToSkip = Random.Range(1, (int)CameraLogic.C_SECTOR_SIZE_Y-1);

        for (int y = 0; y < CameraLogic.C_SECTOR_SIZE_Y; y++)
        {
            if (y == numToSkip) continue;
            Vector3 pos = SectorToPos(sector, x, y - 0.5f);
            pos.z += 1;

            var obj = Instantiate(_wall, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);

        }
    }

    void SpawnVerticalStrip(int sector, int x)
    {

        int numToSkip = -1;

        for (int y = 0; y < CameraLogic.C_SECTOR_SIZE_Y; y++)
        {
            if (y == numToSkip) continue;
            Vector3 pos = SectorToPos(sector, x, y - 0.5f);

            var obj = Instantiate(_wall, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);

        }
    }

    void SpawnPersonOnStrip(int sector, int x)
    {

        int numToSkip = Random.Range(0, (int)CameraLogic.C_SECTOR_SIZE_Y-1);

        for (int y = 0; y < CameraLogic.C_SECTOR_SIZE_Y; y++)
        {
            if (y != numToSkip) continue;
            Vector3 pos = SectorToPos(sector, x, y - 0.5f);

            var obj = Instantiate(_node, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);

        }
    }



    void SpawnWallsWithSingleHole(int sector)
    {
        SpawnVerticalStripWithHole(sector, 0);
        SpawnVerticalStripWithHole(sector, 2);
        SpawnVerticalStripWithHole(sector, 4);
        SpawnVerticalStripWithHole(sector, 6);
    }
    void SpawnWallsWithSingleHoleAndPeople(int sector)
    {
        SpawnVerticalStripWithHole(sector, 0);
        SpawnPersonOnStrip(sector, 1);
        SpawnVerticalStripWithHole(sector, 2);
        SpawnPersonOnStrip(sector, 3);
        SpawnVerticalStripWithHole(sector, 4);
        SpawnPersonOnStrip(sector, 5);
        SpawnVerticalStripWithHole(sector, 6);
        SpawnPersonOnStrip(sector, 7);
    }

    void SpawnWallsWithNoHoleAndPeople(int sector)
    {
        SpawnVerticalStrip(sector, 0);
        SpawnPersonOnStrip(sector, 1);
        SpawnVerticalStrip(sector, 2);
        SpawnPersonOnStrip(sector, 3);
        SpawnVerticalStrip(sector, 4);
        SpawnPersonOnStrip(sector, 5);
        SpawnVerticalStrip(sector, 6);
        SpawnPersonOnStrip(sector, 7);
    }

    void SpawnSolidWalls(int sector)
    {
        SpawnVerticalStrip(sector, 0);
        SpawnVerticalStrip(sector, 1);
        SpawnVerticalStrip(sector, 2);
        SpawnVerticalStrip(sector, 3);
        SpawnVerticalStrip(sector, 4);
        SpawnVerticalStrip(sector, 5);
        SpawnVerticalStrip(sector, 6);
        SpawnVerticalStrip(sector, 7);
    }

    void SpawnTrackers(int sector, float x, float y, float count)
    {

     
        for (int i=0; i < count; i++)
        {
            

            //float y = Random.Range(0, CameraLogic.C_SECTOR_SIZE_Y);

            Vector3 pos = SectorToPos(sector, x, y);
           // pos.z -= 0.2f;

            var obj = Instantiate(_tracker, pos, Quaternion.identity, RTUtil.FindObjectOrCreate("Level").transform);
            x += 0.5f;

        }
    }


      void SpawnSector(int sector)
    {

        Debug.Log("Spawning " + sector);

        switch(sector)
        {

            case 0:
            case 1:
                break;

            case 2:
                SpawnWallsWithSingleHole(sector);
                
                break;
            case 3:
                //recruit
                break;
            case 4:
                SpawnRandom(sector, 0.0f, 0.4f, 0);
    //            SpawnWallsWithSingleHoleAndPeople(sector);
                break;
            case 5:
                //shoot
                break;

            case 6:
                SpawnSolidWalls(sector);
                break;

            case 7:
                break;


            case 8:

                SpawnRandom(sector, 0.0f, 0.0f, 0.3f);
                break;
            case 9:

              
                break;

            case 10:
            case 11:
            case 12:
                SpawnRandom(sector, 0.15f, 0.3f, 0.15f);
                break;

            case 13:

                break;
            case 14:

            
                SpawnTrackers(sector, 0, 0, 4);
                SpawnTrackers(sector, 2, 1, 4);
                SpawnTrackers(sector, 4, 2, 4);
                SpawnTrackers(sector, 6, 3, 4);
                SpawnTrackers(sector, 8, 4, 4);

                break;

            case 15:
               // RTAudioManager.Get().PlayMusic("the_end", 0.8f, 1.0f, false);

                SpawnTrackers(sector, 8, 0, 4);
                SpawnTrackers(sector, 6, 1, 4);
                SpawnTrackers(sector, 4, 2, 4);
                SpawnTrackers(sector, 2, 3, 4);
                SpawnTrackers(sector, 0, 4, 4);

                break;

            case 16:
                SpawnRandom(sector, 0.0f, 0.2f, 0.2f);

                break;

            case 17:

                SpawnTrackers(sector, 8, 0, 4);
                SpawnTrackers(sector, 6, 1, 4);
                SpawnTrackers(sector, 4, 2, 4);
                SpawnTrackers(sector, 2, 3, 4);
                SpawnTrackers(sector, 0, 4, 4);
                SpawnRandom(sector, 0.0f, 0.2f, 0.1f);


                break;
            case 18:

                SpawnSolidWalls(sector);
                SpawnRandom(sector, 0.0f, 0.3f, 0.0f);
                break;

            case 19:
                SpawnRandom(sector, 0.4f, 0.8f, 0.0f);

                break;
            case 20:
              
                break;
            case 21:
                SpawnRandom(sector);
             
                break;
            case 22:
              
                break;

            case 23:
                //all people
                SpawnRandom(sector, 0, 1.0f, 0);

                break;

            case 24:
                //all people
                SpawnRandomSmall(sector, 0, 0, 1.0f);

                break;

            case 25:
               // SpawnRandom(sector, 0, 0.3f, 0);

                break;

            case 29:
                //  CameraLogic.Get().SetSpeed(0);
                //end game


                RTUtil.FindIncludingInactive("Level").BroadcastMessage("Suicide");
                RTMessageManager.Get().Schedule(3, GameLogic.Get().SetGameMode, eGameMode.DEAD);

                break;
            default:

          
                break;
        }


        _lastSpawnedSector = sector;
     //   Debug.Log("Spawning sector " + sector);

    }


    // Update is called once per frame
    void Update()
    {
        //   Debug.Log("CurSec is " + _curSector);

        if (GameLogic.Get()._gameMode != eGameMode.PLAYING) return;
        _curSector =  CameraLogic.Get().GetCurSector();

        if (_lastSpawnedSector <= _curSector)
        {
            SpawnSector(_curSector + 1);
        }

    }
}
